import { Directive, Input, ElementRef, Output, EventEmitter } from '@angular/core';
import { GlobService } from '../service/glob.service';
var HandMoveDirective = (function () {
    function HandMoveDirective(_el, glob) {
        var _this = this;
        this._el = _el;
        this.glob = glob;
        this.offset = new EventEmitter();
        this.documentOffset = new EventEmitter();
        this.moveEnd = new EventEmitter();
        this.parentPos = {};
        this.selfPos = {};
        this.moveDestroy = glob.tool.moveEvent(_el.nativeElement, function (e) {
            _this.documentOffset.emit(e);
        }, function (e) {
            _this.documentOffset.emit(e);
        }, function () { return _this.moveEnd.emit(); });
    }
    HandMoveDirective.prototype.calParent = function () {
        if (!this.parent)
            return;
        var pos = this.glob.pos.getPos(this.parent);
        return Object.assign(this.parentPos, { y0: pos.top, y1: pos.top + pos.h, x0: pos.left, x1: pos.left + pos.w, w: pos.w, h: pos.h });
    };
    HandMoveDirective.prototype.calSelf = function () {
        return Object.assign(this.selfPos, this.glob.pos.getPos(this.self));
    };
    HandMoveDirective.prototype.placeIn = function (x, y) {
        this.self.style.left = x + 'px';
        this.self.style.top = y + 'px';
    };
    HandMoveDirective.prototype.dragMove = function () {
        var _this = this;
        this.glob.pos.posToRel(this.parent);
        this.self.classList.add('moveNode');
        var posP = this.calParent();
        var posS = this.calSelf();
        var x, y, relX, relY;
        this.documentOffset.subscribe(function (e) {
            if (_this.prevent)
                return;
            x = e.x;
            y = e.y;
            relX = posP.x1 - posS.w;
            relY = posP.y1 - posS.h;
            x = x < posP.x0 ? posP.x0 : (x > relX ? relX : x);
            y = y < posP.y0 ? posP.y0 : (y > relY ? relY : y);
            x = x - posP.x0;
            y = y - posP.y0;
            _this.offset.emit({ x: x, y: y });
            if (_this.preventDefault)
                return;
            _this.placeIn(x, y);
        });
        this.resOrder = this.glob.res.valSub.subscribe(function () { return setTimeout(function () { return _this.calParent(); }); });
    };
    HandMoveDirective.prototype.moveTo = function (params, opts) {
        var _this = this;
        if (opts === void 0) { opts = {}; }
        return new Promise(function (resolve) {
            var posP = _this.parentPos, posS = _this.selfPos, bx, by;
            var x = params.x, y = params.y;
            var x0 = _this.self.offsetLeft, y0 = _this.self.offsetTop;
            _this.prevent = true;
            if (typeof x == 'number') {
                x = params.px ? x : posP.w * x - posS.w / 2;
                bx = posP.w - posS.w;
                x = x < 0 ? 0 : (x > bx ? bx : x);
                _this.glob.pos.moveAnimate(Object.assign({ x0: x0, x1: x }, opts), function (num) { return _this.self.style.left = x0 + num + 'px'; }, function () {
                    resolve();
                    _this.prevent = false;
                });
            }
            if (typeof y == 'number') {
                y = params.px ? y : posP.h * y - posS.h / 2;
                by = posP.h - posS.h;
                y = x < 0 ? 0 : (y > by ? by : y);
                _this.glob.pos.moveAnimate(Object.assign({ x0: y0, x1: y }, opts), function (num) { return _this.self.style.top = y0 + num + 'px'; }, function () {
                    if (x === undefined) {
                        resolve();
                        _this.prevent = false;
                    }
                });
            }
            if (typeof x != 'number' && typeof y != 'number')
                _this.prevent = false;
        });
    };
    HandMoveDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.self = this._el.nativeElement;
        this.parent = this.parent || this.self.parentElement;
        this.parent.classList.add('noSelect');
        setTimeout(function () {
            _this.dragMove();
        });
    };
    HandMoveDirective.prototype.ngOnDestroy = function () {
        this.moveDestroy();
        this.resOrder && this.resOrder.unsubscribe();
    };
    HandMoveDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngz-handMove]',
                    exportAs: 'ngz-handMove'
                },] },
    ];
    /** @nocollapse */
    HandMoveDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: GlobService, },
    ]; };
    HandMoveDirective.propDecorators = {
        'offset': [{ type: Output, args: ['offset',] },],
        'documentOffset': [{ type: Output, args: ['documentOffset',] },],
        'moveEnd': [{ type: Output, args: ['moveEnd',] },],
        'preventDefault': [{ type: Input },],
        'parent': [{ type: Input },],
    };
    return HandMoveDirective;
}());
export { HandMoveDirective };
//# sourceMappingURL=hand-move.directive.js.map