import { Component, Input, forwardRef, ElementRef, ChangeDetectionStrategy, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var Pagination = (function () {
    function Pagination(_el, _cd) {
        this._cd = _cd;
        this.autoSkip = true;
        this.type = 'o-p';
        this.pageSize = 10;
        this.size = 5;
        this.btnArr = [];
        this.pages = 5;
        this.maxRange = 1;
        this.el = _el.nativeElement;
    }
    Object.defineProperty(Pagination.prototype, "btnSize", {
        set: function (val) {
            if (!val)
                return;
            var classStr = this.el.className;
            this.el.className = classStr.replace(/btn-group-(sm|lg)/, '');
            val != 'default' && this.el.classList.add('btn-group-' + val);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pagination.prototype, "vertical", {
        set: function (val) {
            if (val) {
                this.el.classList.remove('btn-group');
                this.el.classList.add('btn-group-vertical');
                this._vertical = val;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pagination.prototype, "total", {
        set: function (val) {
            if (!val)
                return;
            this.pages = Math.ceil(val / this.pageSize);
            this.maxRange = Math.ceil(this.pages / this.size);
        },
        enumerable: true,
        configurable: true
    });
    Pagination.prototype.setArr = function (val) {
        var _this = this;
        var min = Math.floor((val - 1) / this.size) * this.size, max = min + this.size;
        if (!this.rotated) {
            if (!this.autoSkip) {
                min++;
                max = max > this.pages ? this.pages : max;
            }
            else {
                min = min === 0 ? 1 : min;
                max = max + 1 > this.pages ? this.pages : max + 1;
            }
        }
        else {
            //rotated:
            var dis = 1 - this.size % 2, len = Math.floor(this.size / 2), leftL = len - dis, rightL = len;
            min = val - leftL;
            if (min < leftL) {
                min = 1;
            }
            else {
                max = val + rightL;
            }
            var over = max - this.pages;
            if (over > 0) {
                min -= over;
                if (min < 1)
                    min = 1;
                max = this.pages;
            }
        }
        //boundaryLinks
        if (this.boundaryLinks) {
            this.bLeft = min == 1;
            this.bRight = max / this.size == this.maxRange;
        }
        if (this.preMin !== min) {
            var arr = [];
            for (var i = min; i <= max; i++) {
                arr.push(i);
            }
            this.btnArr = arr;
        }
        this.preMin = min;
        this._cd.markForCheck();
        setTimeout(function () {
            _this.nowPage = val;
            _this._cd.markForCheck();
        });
    };
    Pagination.prototype.selectPage = function (page) {
        if (page == this.nowPage)
            return;
        this.changeFn(page);
        this.setArr(page);
    };
    Pagination.prototype.registerOnTouched = function () { };
    Pagination.prototype.registerOnChange = function (fn) {
        this.changeFn = fn;
    };
    Pagination.prototype.writeValue = function (val) {
        val && this.setArr(val);
    };
    Pagination.decorators = [
        { type: Component, args: [{
                    selector: 'ngz-pagination',
                    template: "\n        <button (click)=\"selectPage(1)\" [disabled]=\"bLeft||disabled\" *ngIf=\"boundaryLinks\" class=\"btn0 btn-{{type}}\">\n            <a [class.rotate45]=\"_vertical\">\u00AB\u00AB</a>\n        </button>\n        <button [disabled]=\"nowPage==1||disabled\" (click)=\"selectPage(nowPage-1)\" class=\"btn0 btn-{{type}}\">\n            <a [class.rotate45]=\"_vertical\">\u00AB</a></button>  \n        <button [disabled]=\"disabled\" (click)=\"selectPage(i)\" [class.active]=\"i==nowPage\" [ngClass]=\"'btn-'+type\" class=\"btn0\" *ngFor=\"let i of btnArr;\">\n            {{i}}\n        </button>\n        <button [disabled]=\"nowPage==pages||disabled\" (click)=\"selectPage(nowPage+1)\" class=\"btn0 btn-{{type}}\">\n            <a [class.rotate45]=\"_vertical\">\u00BB</a>\n        </button>\n        <button (click)=\"selectPage(pages)\" [disabled]=\"bRight||disabled\" *ngIf=\"boundaryLinks\" class=\"btn0 btn-{{type}}\" style=\"font-size:12px\">\n            <a [class.rotate45]=\"_vertical\">\u00BB\u00BB</a>\n        </button>\n    ",
                    providers: [{
                            provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return Pagination; }), multi: true
                        }],
                    host: {
                        'class': 'btn-group'
                    },
                    changeDetection: ChangeDetectionStrategy.OnPush
                },] },
    ];
    /** @nocollapse */
    Pagination.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ChangeDetectorRef, },
    ]; };
    Pagination.propDecorators = {
        'btnSize': [{ type: Input, args: ['btn-size',] },],
        'vertical': [{ type: Input },],
        'boundaryLinks': [{ type: Input },],
        'autoSkip': [{ type: Input },],
        'type': [{ type: Input },],
        'pageSize': [{ type: Input },],
        'size': [{ type: Input },],
        'rotated': [{ type: Input },],
        'total': [{ type: Input },],
        'disabled': [{ type: Input },],
    };
    return Pagination;
}());
export { Pagination };
//# sourceMappingURL=pagination.component.js.map