import { Component, Input, ContentChildren, forwardRef, Output, EventEmitter } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/throttleTime';
import 'rxjs/add/observable/fromEvent';
import 'rxjs/add/operator/debounceTime';
import { ScrollBindDirective } from './scrollBind.directive';
import { ScrollService } from '../../service/scroll.service';
import { Parent } from './parent';
var ScrollComponent = (function () {
    function ScrollComponent(_scroll) {
        var _this = this;
        this._scroll = _scroll;
        this.baseLine = 0;
        this.throttleTime = 0;
        this.event = new EventEmitter();
        this.scrollOb = Observable.fromEvent(window, 'scroll');
        this.resizeOb = Observable.fromEvent(window, 'resize');
        this.offsetControls = [];
        this.emit = function () { };
        this.changeEmit = function (msn) {
            if (_this.storeMsn !== msn) {
                _this.emit(msn);
                _this.storeMsn = msn;
            }
        };
        this.getControlOffset = function (value) { return _this.offsetControls.find(function (v) { return v.value == value; }); };
        this.scrollToNode = function (node, opts) {
            if (opts === void 0) { opts = {}; }
            opts.value = _this._scroll.pos.getPos(node).top - _this.baseLine;
            return _this.scrollTo2(opts);
        };
    }
    ScrollComponent.prototype.subScroll = function () {
        var _this = this;
        if (this.scrollSub || this.resizeSub)
            return;
        this.scrollSub = this._scroll.scrollEvent.throttleTime(this.throttleTime).subscribe(function (pos) { return !_this.pause && _this.checkOnce(pos); });
        this.resizeSub = this.resizeOb.debounceTime(300).subscribe(function () { return _this.calControls(); });
    };
    ScrollComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.justScroll)
            return;
        //such as inner tab;you need delay calControls;
        setTimeout(function () {
            _this.calControls();
            _this.controls.changes.subscribe(function () { return _this.calControls(); });
            _this.subScroll();
        });
    };
    ;
    ScrollComponent.prototype.getScrollPos = function () {
        var top = this.scrollTop = this._scroll.getScrollTop();
        return { top: top, bottom: top + window.innerHeight };
    };
    ScrollComponent.prototype.checkOnce = function (pos) {
        var i = this.offsetControls.length;
        this.event.emit(pos);
        while (i--) {
            this.bound = this.offsetControls[i].over === 'bottom' ? pos.bottom : pos.top;
            if (this.bound >= this.offsetControls[i].top && this.bound <= this.offsetControls[i].bottom) {
                this.changeEmit(this.offsetControls[i].value);
                return;
            }
        }
        this.changeEmit(undefined);
    };
    ScrollComponent.prototype.calControls = function (refresh) {
        var _this = this;
        var _newOffC = [];
        this.controls.map(function (control) {
            var node = control.el.nativeElement, offsetControl = {};
            var top = (control.remainTop !== undefined && !refresh) ? control.remainTop : _this._scroll.pos.getPos(node).top - _this.baseLine - control.baseLine, controlAttr = {
                top: top,
                bottom: top + node.offsetHeight,
                value: control.value,
                over: control.over
            };
            var existsControl = _this.offsetControls.find(function (v) { return v.value == control.value; });
            if (existsControl) {
                Object.assign(existsControl, controlAttr);
                _newOffC.push(existsControl);
            }
            else {
                _newOffC.push(controlAttr);
            }
            _this.offsetControls = _newOffC;
        });
    };
    ;
    ScrollComponent.prototype.scrollTo = function (opts) {
        this.storeMsn = null;
        var top = this._scroll.getScrollTop(), end = this._scroll.getScrollEnd();
        opts = Object.assign(opts, { top: top, end: end });
        if (opts.value == 'ngz_top') {
            return top <= 0 ? Promise.resolve() : (opts.value = 0) || this.scrollTo2(opts);
        }
        ;
        if (opts.value == 'ngz_bottom') {
            return top >= end ? Promise.resolve() : (opts.value = -1) && this.scrollTo2(opts);
        }
        var offset = this.getControlOffset(opts.value);
        if (!offset)
            return Promise.resolve(false);
        opts.value = this.getControlOffset(opts.value).top + 10;
        return this.scrollTo2(opts);
    };
    ;
    ScrollComponent.prototype.scrollTo2 = function (opts) {
        var _this = this;
        this.pause = true;
        return new Promise(function (resolve) {
            var top = opts.top === undefined ? _this._scroll.getScrollTop() : opts.top, end = opts.end || _this._scroll.getScrollEnd(), pos = opts.value;
            if (pos > end || pos == -1)
                pos = opts.notBottom ? end - 20 : end;
            if (pos < 0)
                pos = 0;
            if (_this.forbidAnimate) {
                _this.pause = false;
                return resolve(!!window.scrollTo(0, pos));
            }
            _this._scroll.pos.moveAnimate({ x0: top, x1: pos, timing: opts.timing, t: opts.duration, vt: opts.vt }, function (n) { window.scrollTo(0, top + n); }, function () { return resolve(_this.pause = false); });
        });
    };
    ScrollComponent.prototype.destroy = function () {
        if (this.scrollSub)
            this.scrollSub.unsubscribe();
        if (this.resizeSub)
            this.resizeSub.unsubscribe();
        this.scrollSub = null;
        this.resizeSub = null;
    };
    ScrollComponent.prototype.registerOnTouched = function () { };
    ;
    ScrollComponent.prototype.registerOnChange = function (fn) {
        this.emit = fn;
    };
    ;
    ScrollComponent.prototype.writeValue = function (value) {
        if (this.pause)
            return;
        value !== undefined && this.offsetControls.length && this.scrollTo({ value: value });
    };
    ;
    ScrollComponent.prototype.ngOnDestroy = function () {
        this.destroy();
    };
    ScrollComponent.decorators = [
        { type: Component, args: [{
                    selector: 'ngz-scroll',
                    template: " <ng-content></ng-content>",
                    providers: [
                        { provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return ScrollComponent; }),
                            multi: true },
                        { provide: Parent, useExisting: forwardRef(function () { return ScrollComponent; }) }
                    ]
                },] },
    ];
    /** @nocollapse */
    ScrollComponent.ctorParameters = function () { return [
        { type: ScrollService, },
    ]; };
    ScrollComponent.propDecorators = {
        'controls': [{ type: ContentChildren, args: [ScrollBindDirective,] },],
        'baseLine': [{ type: Input },],
        'throttleTime': [{ type: Input },],
        'event': [{ type: Output, args: ['event',] },],
        'justScroll': [{ type: Input },],
        'forbidAnimate': [{ type: Input },],
    };
    return ScrollComponent;
}());
export { ScrollComponent };
//# sourceMappingURL=scroll.component.js.map