import { Component, Directive, Input, ViewChild, TemplateRef, ViewContainerRef, ComponentFactoryResolver, ChangeDetectionStrategy, ChangeDetectorRef } from '@angular/core';
import { sideRight } from '../../animate/base';
import { PositionService } from '../../tool/pos.service';
var SideNavComponent = (function () {
    function SideNavComponent(_cd) {
        this._cd = _cd;
    }
    SideNavComponent.prototype.createTp = function (tp) {
        this.container.clear();
        var viewRef = this.container.createEmbeddedView(tp);
        viewRef.context.$implicit = true;
        this._cd.markForCheck();
    };
    SideNavComponent.prototype.clear = function () { this.container.clear(); };
    SideNavComponent.prototype.closeSelf = function () { };
    SideNavComponent.prototype.done = function () {
        this.sideState === 'hid' ? this.closeCb() : this.openCb && this.openCb();
    };
    SideNavComponent.prototype.close = function (cb) {
        this.sideState = 'hid';
        this.closeCb = cb;
        this._cd.markForCheck();
    };
    SideNavComponent.prototype.open = function (cb) {
        this.sideState = 'show';
        this.openCb = cb;
        this._cd.markForCheck();
    };
    SideNavComponent.decorators = [
        { type: Component, args: [{
                    selector: 'ngz-nav-side',
                    template: "\n      <div [class.fadeOut]=\"sideState=='hid'\" class=\"nav-side-bg an_show\">\n      </div>\n      <div (@SideRight.done)=\"done()\" [@SideRight]=\"sideState\" (click)=\"$event.stopPropagation()\" class=\"nav-side-c\" [style.width.px]=\"width\" >\n          <div (click)=\"closeSelf()\" class=\"rel\" [style.height.px]=\"paddingTop\">\n              <span *ngIf=\"!disClose\" class=\"close\" >&times;</span>\n          </div>\n          <div>\n              <a #container></a>\n          </div>\n      </div>\n  ",
                    animations: [sideRight()],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    host: {
                        class: 'nav-side-container',
                        '(click)': 'closeSelf()'
                    }
                },] },
    ];
    /** @nocollapse */
    SideNavComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
    ]; };
    SideNavComponent.propDecorators = {
        'container': [{ type: ViewChild, args: ['container', { read: ViewContainerRef },] },],
        'width': [{ type: Input },],
        'paddingTop': [{ type: Input },],
    };
    return SideNavComponent;
}());
export { SideNavComponent };
var navSideDirective = (function () {
    function navSideDirective(_vcr, _cfr, _pos) {
        var _this = this;
        this._vcr = _vcr;
        this._cfr = _cfr;
        this._pos = _pos;
        this.createTp = function () { return _this.comRef && _this.comRef.instance.createTp(_this.tp); };
    }
    Object.defineProperty(navSideDirective.prototype, "sideTp", {
        set: function (val) {
            this.tp = val;
            val && this.isOpen && this.createTp();
        },
        enumerable: true,
        configurable: true
    });
    navSideDirective.prototype.open = function () {
        var _this = this;
        if (this.prevent || !this.tp || !(this.tp instanceof TemplateRef))
            return;
        this.prevent = true;
        if (this.tp !== this.preTp)
            this.createTp();
        this.preTp = this.tp;
        if (!this.tpEl) {
            var body = document.querySelector('body');
            this.parent = this.parentEl || body;
            this._pos.posToRel(this.parent);
            this.comRef = this._vcr.createComponent(this._cfr.resolveComponentFactory(SideNavComponent));
            this.comRef.instance.createTp(this.tp);
            this.tpEl = this.comRef.location.nativeElement;
            this.tpEl.style.position = this.parent == body ? 'fixed' : (this.forceFix ? 'fixed' : 'absolute');
            this.zIndex && (this.tpEl.style.zIndex = this.zIndex);
            var inst = this.comRef.instance;
            inst.paddingTop = this.top;
            inst.width = this.width;
            inst.disClose = this.disClose;
            inst.closeSelf = this.close.bind(this);
        }
        this.parent.appendChild(this.tpEl);
        this.isOpen = true;
        this.comRef.instance.open(function () { return _this.prevent = false; });
    };
    navSideDirective.prototype.clear = function () {
        this.comRef && this.comRef.destroy();
        this.tpEl = null;
    };
    navSideDirective.prototype.close = function () {
        var _this = this;
        if (this.prevent || !this.isOpen)
            return;
        this.comRef.instance.close(function () {
            _this.parent.removeChild(_this.tpEl);
            _this.prevent = _this.isOpen = false;
        });
    };
    navSideDirective.prototype.toggle = function () {
        this.isOpen ? this.close() : this.open();
    };
    navSideDirective.prototype.ngOnDestroy = function () {
        this.clear();
    };
    navSideDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngz-navSide]',
                    exportAs: 'ngz-navSide'
                },] },
    ];
    /** @nocollapse */
    navSideDirective.ctorParameters = function () { return [
        { type: ViewContainerRef, },
        { type: ComponentFactoryResolver, },
        { type: PositionService, },
    ]; };
    navSideDirective.propDecorators = {
        'sideTp': [{ type: Input, args: ['ngz-navSide',] },],
        'width': [{ type: Input },],
        'forceFix': [{ type: Input },],
        'top': [{ type: Input },],
        'parentEl': [{ type: Input },],
        'zIndex': [{ type: Input },],
        'disClose': [{ type: Input },],
    };
    return navSideDirective;
}());
export { navSideDirective };
//# sourceMappingURL=side-nav.component.js.map