import { Component, ViewChild, Input, forwardRef, ElementRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var SliderComponent = (function () {
    function SliderComponent(el) {
        var _this = this;
        this.el = el;
        this.max = 100;
        this.min = 1;
        this.type = 'p';
        this.getXLen = function () { return _this.thumb.parentPos.h - _this.thumb.selfPos.h; };
        this.getYLen = function () { return _this.thumb.parentPos.w - _this.thumb.selfPos.w; };
        this.moveEnd = function () {
            if (!_this.thumbPos)
                return;
            _this.setValue(_this.vertical ? _this.thumbPos.y : _this.thumbPos.x, _this.vertical ? _this.getXLen() : _this.getYLen());
        };
        this.convertToPos = function (val) { return (val - 1) / (_this.max - _this.min) * (_this.vertical ? _this.getXLen() : _this.getYLen()); };
        this.runCount = 0;
    }
    SliderComponent.prototype.moveThumb = function (e) {
        this.thumbPos = e;
    };
    SliderComponent.prototype.setValue = function (pos, len) {
        this.emit(this.value = pos >= len ? this.max : Math.floor(pos / len * (this.max - this.min)) + this.min);
    };
    SliderComponent.prototype.select = function (e) {
        var _this = this;
        if (this.prevent)
            return;
        this.prevent = true;
        var pos, len, limit, moveParams = { px: true };
        if (this.vertical) {
            len = this.getXLen();
            moveParams.y = pos = e.offsetY >= len ? len : e.offsetY - this.thumb.selfPos.h / 2;
        }
        else {
            len = this.getYLen();
            moveParams.x = pos = e.offsetX >= len ? len : e.offsetX - this.thumb.selfPos.w / 2;
        }
        this.thumb.moveTo(moveParams).then(function () { return _this.prevent = false; });
        pos = pos < 0 ? 0 : pos;
        this.setValue(pos, len);
    };
    SliderComponent.prototype.navTo = function (val) {
        var pos = this.convertToPos(val);
        this.prevent = true;
        this.thumb.moveTo(this.vertical ? { y: pos, px: true } : { x: pos, px: true }).then(this.prevent = false);
    };
    SliderComponent.prototype.registerOnTouched = function () { };
    ;
    SliderComponent.prototype.registerOnChange = function (fn) {
        this.emit = fn;
    };
    ;
    SliderComponent.prototype.initPos = function (val) {
        var container = this.el.nativeElement, child = this.child.nativeElement;
        if (typeof val == 'number') {
            var pos = (val - 1) / (this.max - this.min) * (this.vertical ? container.offsetHeight - child.offsetHeight : container.offsetWidth - child.offsetWidth);
            pos = Math.round(pos) + 'px';
            this.vertical ? (child.style.top = pos) : (child.style.left = pos);
        }
        setTimeout(function () { return child.style.visibility = 'visible'; });
    };
    SliderComponent.prototype.writeValue = function (value) {
        this.runCount++;
        if (this.runCount == 2) {
            return this.initPos(value);
        }
        if (typeof value != 'number' || value < this.min || value > this.max || value === this.value)
            return;
        this.navTo(this.value = value);
    };
    SliderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'ngz-slider',
                    template: "        \n        <div #child (moveEnd)=\"moveEnd()\" (offset)=\"moveThumb($event)\" (click)=\"$event.stopPropagation()\" #thumb=\"ngz-handMove\" class=\"slider-thumb bg-{{type}}\" ngz-handMove></div>\n        <div [ngClass]=\"vertical?'slider-line-vertical':'slider-line'\"></div>\n    ",
                    providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return SliderComponent; }), multi: true }],
                    host: {
                        '[class]': 'vertical?"slider-vertical":"slider"',
                        '(click)': 'select($event)'
                    }
                },] },
    ];
    /** @nocollapse */
    SliderComponent.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    SliderComponent.propDecorators = {
        'max': [{ type: Input },],
        'min': [{ type: Input },],
        'type': [{ type: Input },],
        'vertical': [{ type: Input, args: ['vertical',] },],
        'container': [{ type: ViewChild, args: ['container',] },],
        'child': [{ type: ViewChild, args: ['child',] },],
        'thumb': [{ type: ViewChild, args: ['thumb',] },],
    };
    return SliderComponent;
}());
export { SliderComponent };
//# sourceMappingURL=slider.component.js.map