import { Directive, Input, ElementRef, Output, EventEmitter } from '@angular/core';
import { Resize } from '../service/resize.service';
import { ScrollService } from '../service/scroll.service';
import { ToolService } from '../tool/tool.service';
var StickyBlockDirective = (function () {
    function StickyBlockDirective(res, _scroll, el, tool) {
        var _this = this;
        this.res = res;
        this._scroll = _scroll;
        this.el = el;
        this.tool = tool;
        this.state = new EventEmitter();
        this.relCal = function () { return setTimeout(function () { return _this.calParent(); }); };
    }
    Object.defineProperty(StickyBlockDirective.prototype, "sticky", {
        set: function (val) {
            if (val === undefined)
                return;
            if (val == 'destroy') {
                this.destroy();
            }
            else {
                this.self = this.el.nativeElement;
                this.parent = this.self.parentElement;
                if (!this.parent)
                    return;
                var style = window.getComputedStyle(this.self, null);
                if (!this.useJs && style.position.match(/sticky/)) {
                    this.state.emit('css');
                    this.self.style.top = val + 'px';
                }
                else {
                    this.top = val;
                    this._scroll.pos.posToRel(this.parent);
                    this.state.emit('js');
                    this.setSticky();
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    StickyBlockDirective.prototype.calSelf = function () {
        this.selfPos = this._scroll.pos.getPos(this.self);
        this.stickyTop = this.selfPos.top - this.top;
    };
    StickyBlockDirective.prototype.calParent = function () {
        this.parentPos = this._scroll.pos.getPos(this.parent);
        if (this.selfPos)
            this.stickyBottom = this.parentPos.top + this.parent.clientHeight - this.selfPos.h - this.top;
    };
    StickyBlockDirective.prototype.cal = function () {
        this.calSelf();
        this.calParent();
    };
    StickyBlockDirective.prototype.clearSelf = function () {
        if (!this.self)
            return;
        this.self.style.position = null;
        this.self.style.top = null;
    };
    StickyBlockDirective.prototype.setSticky = function () {
        var _this = this;
        var self = this.self;
        setTimeout(function () {
            _this.cal();
            var handleSticky = function (top) {
                if (top > _this.stickyBottom && !_this.fixed) {
                    self.style.position = 'absolute';
                    self.style.bottom = 0;
                    self.style.top = null;
                }
                else if (top > _this.stickyTop) {
                    self.style.position = 'fixed';
                    self.style.bottom = null;
                    self.style.top = _this.top + 'px';
                }
                else {
                    _this.clearSelf();
                }
            };
            handleSticky(_this._scroll.getPos());
            _this.destroy();
            _this.scrollSub = _this._scroll.scrollEvent.subscribe(function (v) { return handleSticky(v.top); });
            if (_this.detectHeight)
                _this.detectSub = _this._scroll.detectBodyHeight.subscribe(function () { return _this.calParent(); });
        }, 1);
    };
    StickyBlockDirective.prototype.ngOnInit = function () {
    };
    StickyBlockDirective.prototype.destroy = function () {
        this.clearSelf();
        this.scrollSub && this.scrollSub.unsubscribe();
        this.detectSub && this.detectSub.unsubscribe();
    };
    StickyBlockDirective.prototype.ngOnDestroy = function () {
        this.destroy();
    };
    StickyBlockDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[ngz-sticky]',
                    host: {
                        'class': 'sticky'
                    },
                    exportAs: 'ngz-sticky'
                },] },
    ];
    /** @nocollapse */
    StickyBlockDirective.ctorParameters = function () { return [
        { type: Resize, },
        { type: ScrollService, },
        { type: ElementRef, },
        { type: ToolService, },
    ]; };
    StickyBlockDirective.propDecorators = {
        'sticky': [{ type: Input, args: ['ngz-sticky',] },],
        'useJs': [{ type: Input },],
        'state': [{ type: Output },],
        'detectHeight': [{ type: Input },],
        'fixed': [{ type: Input },],
    };
    return StickyBlockDirective;
}());
export { StickyBlockDirective };
//# sourceMappingURL=sticky-block.directive.js.map